% This demo shows an example for edge-wise group analysis (group0 vs. group1) 
% for a population of synthetic graphs (e.g., brain network). 
%
% Code written by Won Hwa Kim, won.kim@uta.edu
% Updated: May 24, 2018

%% Initialization
clc;
clear;
close all;

rng(0);

n = 40; % number of subjects for each group
N = 20;  % number of vertices
siz = [N,N]; % size of adj matrix
sigma = .3; % variance for noise

%% parameters for SGWT
Nscales = 5;  
designtype = 'abspline3'; % kernel type
lpfactor = 1;

%% generate a template graph
fprintf('generating a random graph with %d vertices...\n', N);
density = 0.2; % a rough estimate of the amount of edges       
[G, id] = mbca_generate_random_A(N, density); % lower triangular adj matrix G

%% Generating synthetic data
% Generate A0 and A1 which are cell arrays of adjacency matrices...
n_vtx = length(id);  % number of non-zero elements in G

mu0 = zeros(siz); 
for i=1:length(id);
    mu0(id(i,1), id(i,2)) = 2; % mean for edge weights for group 0
end

mu1 = zeros(siz);
for i=1:length(id)    
    mu1(id(i,1), id(i,2)) = 1.8; % mean for edge weights for group 1
end

% ground truth: differences between mu0 and mu1
gt = abs(mu0-mu1);
gt = gt+gt';
figure;imagesc(gt);title('ground truth');colorbar;

% generate a population of adj matrices based on mu0 and mu1
fprintf('generating %d adj matrices for group 0... \n', n);
A0 = cell(n,1);
for i=1:n
    % for group 0
    A = mu0;    
    r = sigma*randn(n_vtx,1); % Gaussian noise
    for j=1:n_vtx
        A(id(j,1), id(j,2)) = A(id(j,1), id(j,2)) + r(j); % adding noise ~N(0,1) for each edge
    end
    A = A+A'; % make a symmetric adj matrix
    A0{i} = A;
end

fprintf('generating %d adj matrices for group 1... \n', n);
A1 = cell(n,1);
for i=1:n
    %for group 1
    A = mu1;    
    r = sigma*randn(n_vtx,1); % Gaussian noise
    for j=1:n_vtx
        A(id(j,1), id(j,2)) = A(id(j,1), id(j,2)) + r(j); % adding noise ~N(0,1) for each edge
    end
    A = A+A'; % make a symmetric adj matrix
    A1{i} = A;
end

%% SGWT parameters

% largest eigenvalue to define a sgwt kernel
tmp = mu0+mu0';
L1 = diag(sum(tmp)) - tmp; 
lmax1 = sgwt_rough_lmax(L1);

tmp = mu1+mu1';
L2 = diag(sum(tmp)) - tmp; 
lmax2 = sgwt_rough_lmax(L1);

lmax = max(lmax1, lmax2);

% define kernels in the freq space
arange = [0 lmax];
[g,t]=sgwt_filter_design(lmax,Nscales,'designtype',designtype,'lpfactor',lpfactor);
 
% figure;
% sgwt_view_design(g, t, arange);

m=30;   % kernel approximation poly order
c = cell(numel(g),1);
for k=1:numel(g)
    c{k}=sgwt_cheby_coeff(g{k},m,m+1,arange);
end

J = length(t);
%% computing multi-resolution decriptors (WaCS)

% group 0
W0 = cell(n,1);
fprintf('Group 0: deriving WaCS from the graph...\n')    
for k = 1:n
    %fprintf('Group 0: deriving WaCS from the %d-th graph...\n', k)    
    NW = A0{k};  % a graph (or network)
    
    [LG, f, idx, n_vertex]= mbca_LGT(NW);  % line graph transform
    L = sgwt_laplacian(LG);   % Laplacian of L(G)
    
    w = sgwt_cheby_op(f, L, c, arange);  % wavelet transform

    W0{k} = cell(J,1);
    for s=1:J        
        W0{k}{s} = mbca_inverseLGT(w{s},idx,N); % back to the original structure 
    end   
end

% group 1
W1 = cell(n,1);
fprintf('Group 1: deriving WaCS from the graph...\n')    
for k=1:n
    %fprintf('Group 1: deriving WaCS from the %d-th graph...\n', k)     
    NW = A1{k};
    
    [LG, f, idx, n_vertex]= mbca_LGT(NW);  % line graph transform
    L = sgwt_laplacian(LG);  % Laplacian of L(G)
    
    w = sgwt_cheby_op(f, L, c, arange); % wavelet transform

    W1{k} = cell(J,1);
    for s=1:J        
        W1{k}{s} = mbca_inverseLGT(w{s},idx,N); % back to the original structure
    end   
end

%% vectorize WaCS for hypothesis testing...
% group 0
WaCS0 = mbca_vectorize_WaCS(W0);    
% group 1
WaCS1 = mbca_vectorize_WaCS(W1);

%% Multivariate statistical analysis (Hotelling's T2 test)
d = 1:4;    %using scale 1 to 4
[h, p_wacs] = hotelT2(WaCS0(:,d,:), WaCS1(:,d,:));
p_wacs(isnan(p_wacs)) = 1;  % disgard NaNs

n_test = (N*N-N)/2;
[p_corrected, thres] = mbca_Bonferroni(p_wacs, 0.05, n_test);

figure;imagesc(reshape(-log10(p_corrected),siz));caxis([0 10]);colorbar;
title('Using WaCS: p-value in -log\_10')

%% Statistical analysis on the raw data
for i=1:n
    NW0(:,i) = A0{i}(:);    
end
for i=1:n
    NW1(:,i) = A1{i}(:);    
end

[h, p_raw] = ttest2(NW0', NW1');
p_raw(isnan(p_raw)) = 1;
[p_corrected, thres] = mbca_Bonferroni(p_raw, 0.05, n_test);

figure;imagesc(reshape(-log10(p_corrected),siz));caxis([0 10]);colorbar;
title('Using raw data: p-value in -log\_10')
